/*
	TunesTEXT.js
		Dashboard Widget TunesTEXT by cyanworks.net
*/

// AppleScrollbar
var scrollBar;
var scrollArea;

// Objects
var styleCollection;
var widgetPref;
var lyricsDatabase
var saveReminder;
var currentTrack;
var frontDisplay;
var widgetWindow;
var growBox;
var lyricsEditor;
var updateChecker;

// Buttons
var iTunesControl;
var iTunesBack;
var iTunesPlay;
var iTunesNext;
var artworkControl;
var artworkButton;

var display;
var backsidePane;
var websiteURI = "http://www.cyanworks.net/dboardTunesTEXT.html";

// Initialize widget
function init() {
	if (window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
	}

	// Plugin
	if (window.widget && TunesTEXTPlugin)
		LOG__("Widget plugin loaded.");
	else
		alert("Widget plugin not loaded.");

	// AppleScrollbar
	scrollBar  = new AppleVerticalScrollbar(document.getElementById("myScrollBar"));
	scrollArea = new AppleScrollArea(document.getElementById("contentBox"), scrollBar);
	scrollArea.scrollsHorizontally = false;
	scrollArea.scrollsHorizontally = true;
//	scrollArea.singlepressScrollPixels = 25;
	scrollArea.focus();
	scrollArea.reveal(document.getElementById("lyrics"));
	scrollArea.refresh();
	window.onfocus = onFocus;
	window.onblur  = onBlur;

	// Buttons
	iTunesControl = new Fader("iTunesControl", false, "front");
	iTunesBack = new Button("iTunesBack",
					"Images/iTunesBack.png", "Images/iTunesBackClicked.png", null);
	iTunesPlay = new Button("iTunesPlay",
					"Images/iTunesPlay.png", "Images/iTunesPlayClicked.png", null);
	iTunesNext = new Button("iTunesNext",
					"Images/iTunesNext.png", "Images/iTunesNextClicked.png", null);
	artworkControl = new Fader("artworkControl", false, "front");
	artworkButton = new Button("artworkButton",
					"Images/artworkButton.png", "Images/artworkButtonClicked.png", null);

	// Objects
	styleCollection = new StyleCollection();
	widgetPref      = new WidgetPreference();
	lyricsDatabase  = new LyricsDatabase();
	saveReminder    = new SaveReminder();
	currentTrack    = new CurrentTrack();
	widgetWindow    = new WidgetWindow(document.getElementById("growBox"));
	frontDisplay    = new FrontDisplay(currentTrack);
	lyricsEditor    = new LyricsEditor();
	updateChecker   = new UpdateChecker();
	updateChecker.checkFinished = function() { updateCheckFinished(); };

	setDisplay(widgetPref.value("display"));
	setBacksidePane(widgetPref.value("backsidePane"));

	// Buttons action
	iTunesBack.onClick = function() { TunesTEXTPlugin.iTunesControl("back track"); };
	iTunesPlay.onClick = function() { TunesTEXTPlugin.iTunesControl("playpause"); };
	iTunesNext.onClick = function() { TunesTEXTPlugin.iTunesControl("next track"); };
	artworkButton.onClick = function() {
		if (display != "compact")
			frontDisplay.setDisplayArtwork("++");
		else
			setDisplay("regular");
	};

	// Events
	document.addEventListener("keyup",   onKeyUp,   true);
	document.addEventListener("keydown", onKeyDown, true);

	// Localizing
	localizeInnerText();

	// Display version number
	displayVersionNumber();
	
	// Start Update Checker
	updateChecker.start();

	currentTrack.update(true, null, null);
}

function onShow() {
}

function onHide() {
}

function onFocus() {
	if (display != "backside")
		scrollArea.focus();
}

function onBlur() {
	scrollArea.blur();
}

function search() {
	currentTrack.search();
}

function download(sourceNum) {
	currentTrack.downloadLyrics(sourceNum);
}

function save(event) {
	currentTrack.save();
}

function clear(event) {
	currentTrack.lyrics = "";
	document.getElementById("textView").value = currentTrack.lyrics;
}

function getLyrics(event, sourceNum, uriForBrowser) {
    if (!event.metaKey)
        download(sourceNum);
    else
        openWebsite(uriForBrowser);
}

function iTunesUpdated(playerState, title, artist, album) {
	LOG__("iTunesUpdated:" + playerState + ", " + title + ", " + artist + ", " + album);
	if (title && title != "")
		currentTrack.update(false, title, artist, album);
}

function statusUpdated(status) {
	LOG__("statusUpdated");
	frontDisplay.setStatus(status, "", true);
}

function updateCheckFinished() {
	LOG__("updateCheckFinished");
	var result = updateChecker.judge();
	if (result == -1)
		frontDisplay.setStatus("A new version of TunesTEXT is available.", "", true);
}

function displayVersionNumber() {
	LOG__("displayVersionNumber");
    
    var versionDiv = document.getElementById("version");
    var ver = updateChecker.myVersion;
    var innerStr = versionDiv.innerHTML;
    
	if (ver != null)
		versionDiv.innerHTML = innerStr.replace("%version%", ver);
	else
		versionDiv.innerHTML = innerStr.replace("%version%", "");
}

function onKeyUp(event) {
	var handled = true;
	if (display == "backside") return;
	LOG2__("onKeyUp charCode:" + event.charCode);

	switch(event.charCode) {
		case 115:	// s
		case 83:	// S
			if (!event.shiftKey)
				frontDisplay.setStyleNum("++");
			else
				frontDisplay.setStyleNum("--");
			break;
		case 59:	// +
		case 43:	// +
			frontDisplay.setFontSize("++");
			break;
		case 45:	// -
			frontDisplay.setFontSize("--");
			break;
		case 63234:	// left
			iTunesBack.keyUp();
			break;
		case 32:	// space
			iTunesPlay.keyUp();
			break;
		case 63235:	// right
			iTunesNext.keyUp();
			break;
		case 103:	// g
		case 71:	// G
			artworkButton.keyUp();
			break;
		case 119:	// w
		case 87:	// W
			openWebsite(websiteURI);
			break;
		case 102:	// f
		case 70:	// F
			widgetWindow.resizeToFit();
			break;
		case 46:	// .
			TunesTEXTPlugin.cancel();
			break;
		case 117:	// u
		case 85:	// U
			updateChecker.check();
			break;
		default:
			handled = false;
			break;
	}

	if (handled) {
		event.stopPropagation();
		event.preventDefault();
	}
}

function onKeyDown(event) {
	LOG2__("onKeyDown charCode:" + event.charCode);
	var handled = true;

	if (display != "backside") {
		switch(event.charCode) {
			case 63234:	// left
				iTunesBack.keyDown();
				break;
			case 32:	// space
				iTunesPlay.keyDown();
				break;
			case 63235:	// right
				iTunesNext.keyDown();
				break;
			case 103:	// g
			case 71:	// G
				artworkButton.keyDown();
				break;
			case 115:	// s
			case 83:	// S
			case 59:	// +
			case 43:	// +
			case 45:	// -
			case 119:	// w
			case 87:	// W
			case 102:	// f
			case 70:	// F
			case 46:	// .
				break;
			default:
				handled = false;
				break;
		}
	} else {
		if (event.metaKey && !(saveReminder.active)) {
			switch(event.charCode) {
				case 115:	// s
					currentTrack.save();
					break;
				case 122:	// z
				case 90:	// Z
					if (!event.shiftKey)
						TunesTEXTPlugin.undo();
					else
						TunesTEXTPlugin.redo();
					break;
				default:
					handled = false;
					break;
			}
		} else {
			handled = false;
		}
	}

	if (handled) {
		event.stopPropagation();
		event.preventDefault();
	}
}

function searchFinished(searchResult) {
	LOG__("searchFinished is notified");
	lyricsEditor.loadSearchResult(searchResult);
}

function downloadFinished(downloadedLyrics) {
	LOG__("downloadFinished is notified");
	currentTrack.loadDownloadedLyrics(downloadedLyrics);
}

function downloadError(error) {
	frontDisplay.setStatus("Download error.", "", true)
}

// ================================================
// WidgetWindow
// ================================================
function WidgetWindow(growBox) {
	LOG__("Object: WidgetWindow");
	this.width = 280;
	this.height = 400;
	this.minWidth = 280;
	this.minHeight = 130;
	this.widthRev = 320;
	this.heightRev = 480;
	this.setAutoResize(widgetPref.value("autoResize"));

	var localThis = this;
	this.mouseDownHandler = function(event) { localThis.resizeStart(event); };
	this.mouseMoveHandler = function(event) { localThis.resizeTrack(event); };
	this.mouseUpHandler   = function(event) { localThis.resizeEnd(event); };

	this.growBox = growBox;
	this.growBox.addEventListener("mousedown", this.mouseDownHandler, false);

	// Restore last window size
	if (this.autoResize == false) {
		this.resizeTo(widgetPref.value("widgetWidth"), widgetPref.value("widgetHeight"));
		this.resizeFinished();
	}
}

WidgetWindow.prototype.setAutoResize = function(value) {
	LOG__("WidgetWindow.setAutoResize(" + value + ")");
	var newValue = false; // Default
	if ((value == true) || (value == false))
		newValue = value

	this.autoResize = newValue;
	document.getElementById("autoResizeCheck").checked = newValue;
	widgetPref.setValue(newValue, "autoResize");
}

WidgetWindow.prototype.resizeTo = function(width, height) {
	var front = document.getElementById("front");
	var newWidth = width;
	var newHeight = height;
	var newWidthMax = window.screen.width - window.screenX;
	var newHeightMax = window.screen.height - window.screenY - 48;

	if (width != null) {
		if (width < this.minWidth)
			newWidth = this.minWidth;
/*
		if (newWidthMax < width)
			newWidth = newWidthMax;
*/
	} else {
		newWidth = 280;
	}

	if (height != null) {
		if (height < this.minHeight)
			newHeight = this.minHeight;
/*
		if (newHeightMax < height)
			newHeight = newHeightMax;
*/
	} else {
		newHeight = 400;
	}

	this.width = newWidth;
	this.height = newHeight;

	front.style.width = newWidth + "px";
	front.style.height = newHeight + "px";

	if (window.widget)
		window.resizeTo(newWidth, newHeight);

	scrollArea.refresh();
}

// Fit window size to lyrics
WidgetWindow.prototype.resizeToFit = function() {
	LOG2__("WidgetWindow.resizeToFit");

	var front        = document.getElementById("front");
	var lyricsDiv    = document.getElementById("lyrics");
	var lyricsStyle  = document.defaultView.getComputedStyle(lyricsDiv,"");
	var lyricsWidth  = parseInt(lyricsStyle.getPropertyValue("width"));
	var lyricsHeight = parseInt(lyricsStyle.getPropertyValue("height"));
	var goalLyricsHeight = 400 - 98;

	var newWidth = this.minWidth;
	var newHeight = this.minHeight;
	var newWidthMax = window.screen.width - window.screenX;
	var newHeightMax = window.screen.height - window.screenY - 48;

	// Spread window and measure height of lyrics
	front.style.height = newHeight + "px";
	front.style.width = newWidthMax + "px";
	goalLyricsHeight = lyricsDiv.scrollHeight;

	// New width of window
	front.style.width = newWidth + "px";
	lyricsHeight = lyricsDiv.scrollHeight;

	while ((goalLyricsHeight < lyricsHeight) && (newWidth <= newWidthMax)) {
		newWidth += 5;
		front.style.width = newWidth + "px";
		lyricsHeight = lyricsDiv.scrollHeight;
	}

	// New height of window
	if (frontDisplay && frontDisplay.displayArtwork) {
		if (goalLyricsHeight + 84 + 150 <= newHeightMax)
			newHeight = goalLyricsHeight + 84 + 150;
		else
			newHeight = newHeightMax;
	} else {
		if (goalLyricsHeight + 84 <= newHeightMax)
			newHeight = goalLyricsHeight + 84;
		else
			newHeight = newHeightMax;
	}

	if (newHeight < this.minHeight)
		newHeight = this.minHeight;

	this.resizeTo(newWidth, newHeight);
	this.resizeFinished();
}

WidgetWindow.prototype.resizeStart = function(event) {
	LOG2__("WidgetWindow.resizeStart(" + this.width + ", " + this.height + ")");
	this.lastPointX = event.screenX;
	this.lastPointY = event.screenY;

	document.addEventListener("mousemove", this.mouseMoveHandler, true);
	document.addEventListener("mouseup", this.mouseUpHandler, true);
	event.stopPropagation();
	event.preventDefault();
}

WidgetWindow.prototype.resizeTrack = function(event) {
	var minWidth      = this.minWidth;
	var minHeight     = this.minHeight;
	var newWidth      = this.width;
	var newHeight     = this.height;
	var newLastPointX = this.lastPointX;
	var newLastPointY = this.lastPointY;

	LOG2__("WidgetWindow.resizeTrack(" + newWidth + "," + newHeight
			+ ") screen(" + event.screenX + "," + event.screenY
			+ ") last(" + newLastPointX + "," + newLastPointY + ")");

	if (minWidth <= newWidth + (event.screenX - newLastPointX)) {
		newWidth += (event.screenX - newLastPointX);
		newLastPointX = event.screenX;
	} else {
		newWidth = minWidth;
	}
	
	if (minHeight <= newHeight + (event.screenY - newLastPointY)) {
		newHeight += (event.screenY - newLastPointY);
		newLastPointY = event.screenY;
	} else {
		newHeight = minHeight;
	}

	this.lastPointX = newLastPointX;
	this.lastPointY = newLastPointY;
	this.resizeTo(newWidth, newHeight);

	event.stopPropagation();
	event.preventDefault();
}

WidgetWindow.prototype.resizeEnd = function(event) {
	LOG2__("WidgetWindow.resizeEnd(" + this.width + "," + this.height + ")");
	document.removeEventListener("mousemove", this.mouseMoveHandler, true);
	document.removeEventListener("mouseup", this.mouseUpHandler, true);
	event.stopPropagation();
	event.preventDefault();
	this.resizeFinished();
}

WidgetWindow.prototype.resizeFinished = function() {
	LOG__("WidgetWindow.resizeFinished(" + this.width + ", " + this.height + ")");
	widgetPref.setValue(this.width, "widgetWidth");
	widgetPref.setValue(this.height, "widgetHeight");
}

WidgetWindow.prototype.setMinHeight = function(value) {
	LOG__("WidgetWindow.setMinHeight(" + value + ")");

	this.minHeight = value;
	if (this.height < this.minHeight)
		this.resizeTo(this.width, this.minHeight)
	this.resizeFinished();
}


// ================================================
// LyricsDatabase
// ================================================
function LyricsDatabase() {
	LOG__("Object: LyricsDatabase");

	this.setDatabases(widgetPref.value("useUtamap"),  "utamap");
	this.setDatabases(widgetPref.value("useMetro"),   "metro");
	this.setDatabases(widgetPref.value("useSing365"), "sing365");
}

LyricsDatabase.prototype.setDatabases = function(value, theDatabase) {
	LOG__("LyricsDatabase.setDatabases(" + value + ", " + theDatabase + ")");
	var newValue;

	switch(theDatabase) {
		case "utamap":
			newValue = true; // Default
			break;
		case "metro":
			newValue = true; // Default
			break;
		case "sing365":
			newValue = false; // Default
			break;
		default:
			return;
			break;
	}

	if ((value == true) || (value == false))
		newValue = value

	switch(theDatabase) {
		case "utamap":
			this.useUtamap = newValue;
			document.getElementById("useutamap").checked = newValue;
			TunesTEXTPlugin.setUseUtamap(newValue);
			widgetPref.setValue(newValue, "useUtamap");
			break;
		case "metro":
			this.useMetro = newValue;
			document.getElementById("usemetro").checked = newValue;
			TunesTEXTPlugin.setUseMetro(newValue);
			widgetPref.setValue(newValue, "useMetro");
			break;
		case "sing365":
			this.useSing365 = newValue;
			document.getElementById("usesing365").checked = newValue;
			TunesTEXTPlugin.setUseSing365(newValue);
			widgetPref.setValue(newValue, "useSing365");
			break;
	}
}

LyricsDatabase.prototype.databasesCheckOnChange = function(event) {
	var flag = event.target.checked;
	switch(event.target.id) {
		case "useutamap":
			this.setDatabases(flag, "utamap");
			break;
		case "usemetro":
			this.setDatabases(flag, "metro");
			break;
		case "usesing365":
			this.setDatabases(flag, "sing365");
			break;
	}
}

// ================================================
// SaveReminder
// ================================================
function SaveReminder() {
	this.textView = document.getElementById("textView");
	this.reminder = document.getElementById("reminder");
	this.lyrics = null;
	this.active = false;
}

SaveReminder.prototype.reset = function() {
	/*
		There is a bug in WebKit (Mac OS X 10.4.9 / Safari 2.0.4)
			Bug 12370 - Assigning a value to a hidden textarea reverts textarea to default value
			http://bugs.webkit.org/show_bug.cgi?id=12370
			(closed as RESOLVED/FIXED)
	*/
	if (display != "backside") {
		alert("Can't get the value of hidden textarea.");
		return false;
	}

	this.lyrics = this.textView.value;
}

SaveReminder.prototype.check = function() {
	var oldLyrics, currentLyrics;
	var dirty;
	
	if (display != "backside") {
		alert("Can't get the value of hidden textarea.");
		return false;
	}

	oldLyrics = this.lyrics;
	currentLyrics = this.textView.value;

	if (oldLyrics != null) {
		oldLyrics     = oldLyrics.replace(/\r\n/g, "\n");
		oldLyrics     = oldLyrics.replace(/\r/g, "\n");
		currentLyrics = currentLyrics.replace(/\r\n/g, "\n");
		currentLyrics = currentLyrics.replace(/\r/g, "\n");
		dirty = (oldLyrics != currentLyrics);
	} else {
		dirty = (currentLyrics != "");
	}

	if (dirty) {
		this.run();
		return true;
	} else {
		return false;
	}
}

SaveReminder.prototype.run = function() {
	this.active = true;
	this.reminder.style.display = "block";
}

SaveReminder.prototype.quit = function() {
	this.active = false;
	this.reminder.style.display = "none";
}

SaveReminder.prototype.noSave = function() {
	this.quit();
	this.reset();
	showFront();
}

SaveReminder.prototype.cancel = function() {
	this.quit();
}

SaveReminder.prototype.save = function() {
	currentTrack.save();
	this.quit();
	showFront();
}

// ================================================
// CurrentTrack
// ================================================
function CurrentTrack() {
	LOG__("Object: CurrentTrack");

	var useEL = widgetPref.value("useEmbeddedLyrics");
	if ((useEL == true) || (useEL == false))
		this.setUseEmbeddedLyrics(useEL);
	else
		this.setUseEmbeddedLyrics(true);

	this.clear();
}

CurrentTrack.prototype.update = function(force, title, artist, album) {
	if (display == "backside") return;
	LOG__("CurrentTrack.update(" + force + ", " + title + ", " + artist + ", " + album + ")");
	var nowTrack, trackData, lyricsHTML, child;

	// No update
	if (title && artist && !force) {
		if ((title == this.title) && (artist == this.artist)) {
            if (album == this.album)
                return;
        }
    }

	// Recieve track data from iTunes
	nowTrack = TunesTEXTPlugin.trackWS();
	// Cannot get track data
	if (!nowTrack || !nowTrack[0] || nowTrack[0] == "" || nowTrack[0] == "undefined") {
		LOG__("Cannot get track data.");
		this.clear();
		TunesTEXTPlugin.cancel();
		return;
	}
	// No update
	if ((this.title == nowTrack[0]) && (this.artist == nowTrack[1]) && (this.album == nowTrack[2]) && !force)
		return;

	// Update track data
	frontDisplay.setStatus("", "", false);
	TunesTEXTPlugin.cancel();
	this.title            = nowTrack[0];
	this.artist           = nowTrack[1];
	this.album            = nowTrack[2];
	this.source           = nowTrack[3];
	this.videoKind        = nowTrack[4];
	this.location         = nowTrack[5];
	this.artwork          = nowTrack[6];
	this.trackID          = nowTrack[7];
	this.lyricsEmbedded   = nowTrack[8];
	this.lyricsOnTTFile   = nowTrack[9];	// If the file does not exist, this value is null.
	this.lyrics = "";

	// Check the source of music
	if (this.source && ((this.source == "radio") || (this.source == "podcast"))) {
		LOG__("Source:" + this.source);
		frontDisplay.setTrack(this);
		return;
	}
	// Check lyrics (provisional)
	this.checkLyrics(true);

	LOG__("CurrentTrack.update: " + this.title + ", " + this.artist + ", " + this.album + ", "
			+ this.source + ", " + this.videoKind + ", " + this.location + ", " + this.trackID + ", "
			+ ((this.lyrics.length <= 20) ?
				this.lyrics : (this.lyrics.replace(/[\n\r]/, " ").substr(0,16) + "...")) + ", "
			+ ((this.artwork != "") ?
				(this.artwork.replace(/[\n\r]/, " ").substr(0,22) + "...") : "No artwork"));

	// Check lyrics
	this.checkLyrics(false);
}

CurrentTrack.prototype.checkLyrics = function(provisional) {
	if (this.useEmbeddedLyrics)
		this.checkLyricsEmbedded(provisional);
	else
		this.checkLyricsOnTTFile(provisional);

	frontDisplay.setTrack(this);
	if (!provisional)
		document.getElementById("textView").value = this.lyrics.plainToForm();
}

CurrentTrack.prototype.checkLyricsEmbedded = function(provisional) {
	LOG__("CurrentTrack.checkLyricsEmbedded");

	var isEmbedded = ((this.lyricsEmbedded != null)
						&& (this.lyricsEmbedded != "") && (this.lyricsEmbedded != "msng"));
	var isFileExists = (this.lyricsOnTTFile != null);

	// Use Embedded lyrics
	if (isEmbedded)
		this.lyrics = this.lyricsEmbedded;

	// Try to use lyrics in TunesTEXT data file
	if (!isEmbedded && isFileExists) {
		if (this.lyricsOnTTFile != "") {
			this.lyrics = this.lyricsOnTTFile;
			if (!provisional) {
				if (TunesTEXTPlugin.embedLyrics(this.lyrics))
					frontDisplay.setStatus("Lyrics embedded in iTunes.", "", false);
			}
		} else {
			this.lyrics = "";
		}
	}

	// Search lyrics
	if (!isEmbedded && !isFileExists) {
		this.lyrics = "";

		// If movie or TV show is playing, do not search lyrics.
		if ((this.videoKind == "movie") || (this.videoKind == "TV show"))
			return;

		if (!provisional) {
			frontDisplay.setStatus("No lyrics. Start WWW search.", "", false);
			this.search();
			return;
		}
	}
}

CurrentTrack.prototype.checkLyricsOnTTFile = function(provisional) {
	LOG__("CurrentTrack.checkLyricsOnTTFile");

	var isFileExists = (this.lyricsOnTTFile != null);

	// Use lyrics in TunesTEXT data file
	if (isFileExists)
		this.lyrics = this.lyricsOnTTFile;

	// Search lyrics
	if (!isFileExists) {
		this.lyrics = "";
		if (!provisional) {
			frontDisplay.setStatus("No lyrics. Start WWW search.", "", false);
			this.search();
			return;
		}
	}
}

CurrentTrack.prototype.save = function() {
	var embedSuccess = false;

	// Load lyrics from Editor
	if (display == "backside") {
		TunesTEXTPlugin.cancel();
		this.lyrics = document.getElementById("textView").value;
	}

	// Embed lyrics in music file if useEmbeddedLyrics option is ON, 
	if (this.useEmbeddedLyrics) {
		if (TunesTEXTPlugin.embedLyrics(this.lyrics)) {
			embedSuccess = true;
			LOG__("Lyrics embeded in iTunes.");
		} else {
			alert("Failed to embed Lyrics in iTunes.");
		}
	}

	// Save lyrics in TunesTEXT data file
	if (TunesTEXTPlugin.saveLyrics(this.lyrics)) {
		if (this.lyrics != "") {
			if (embedSuccess)
				frontDisplay.setStatus("File saved, and lyrics embeded in iTunes.", "", false);
			else
				if (this.useEmbeddedLyrics) 
					frontDisplay.setStatus("Failed to embed lyrics in iTunes.", "", true);
				else
					frontDisplay.setStatus("File saved.", "", false);
		} else {
			if (embedSuccess)
				frontDisplay.setStatus("Empty file saved, and cleared lyrics in iTunes.", "", false);
			else
				frontDisplay.setStatus("Empty file saved.", "", false);
		}
		this.update(true);
	} else {
		alert("Failed to save the file.");
	}
	
	if (display == "backside")
		saveReminder.reset();
}

CurrentTrack.prototype.search = function() {
    LOG__("CurrentTrack.search()");
    var theTitle;
    var theArtist;
    var formTitle = document.getElementById("searchTitle").value;
    var formArtist = document.getElementById("searchArtist").value;

	if (!this.title || this.title == "" || this.title == "undefined"
			|| !this.artist || this.artist == "" || this.artist == "undefined") {
		frontDisplay.setStatus("WWW Search is canceled. No title or artist name.", "", false);
		return null;
	}

    theTitle = (formTitle == "")? this.title : formTitle;
    theArtist = (formTitle == "")? this.artist : formArtist;

	lyricsEditor.clearSearchResult();
	TunesTEXTPlugin.searchLyrics(theTitle, theArtist);
}

CurrentTrack.prototype.evaluateSearchResult = function() {
	LOG__("CurrentTrack.evaluateSearchResult");
	var downloadUri;
	var found = false;
	var i;

	if (lyricsEditor.searchResult.length <= 0) {
		if (display != "backside") {
			// Save empty data file
			this.lyrics = "";
			this.save();
		}
		return;
	}

	if (display != "backside") {
		for (i = 0; i < lyricsEditor.searchResult.length; i++) {
			if (1 < lyricsEditor.searchResult[i].score - 0) {
				downloadUri = lyricsEditor.searchResult[i].uri;
				found = true;
				break;
			}
		}
		if (found) {
			// Download lyrics
			this.downloadLyrics(downloadUri);
		} else {
			// Save empty data file
			this.lyrics = "";
			this.save();
		}
	}
}

CurrentTrack.prototype.downloadLyrics = function(sourceNum) {
	frontDisplay.setStatus("Downloading lyrics.", "", true);
	TunesTEXTPlugin.downloadLyrics(sourceNum);
}

CurrentTrack.prototype.loadDownloadedLyrics = function(downloadedLyrics) {
	if (this.lyrics = downloadedLyrics) {
		if (display != "backside") {
            // Frontside : Display lyrics immediately and save it
            document.getElementById("textView").value = this.lyrics.plainToForm();
			this.save();
        } else {
            // Backside : Delete current text and display lyrics later
            document.getElementById("buffer").value = this.lyrics.plainToForm();
            document.getElementById("textView").value = "";
            setTimeout ("lyricsEditor.loadBuffer();", 100);
        }
		frontDisplay.setStatus("Finished download lyrics.", "", false);
	} else {
		frontDisplay.setStatus("Failed to download lyrics.", "", true);
		if (display != "backside") {
			this.lyrics = "";
			this.save();
		}
	}
}

CurrentTrack.prototype.clear = function() {
	this.title          = null;
	this.album          = null;
	this.artist         = null;
	this.source         = null;
	this.location       = null;
	this.artwork        = null;
	this.trackID        = null;
	this.lyricsEmbedded = null;
	this.lyricsOnTTFile = null;
}

CurrentTrack.prototype.setUseEmbeddedLyrics = function(value) {
	LOG__("CurrentTrack.setUseEmbeddedLyrics(" + value + ")");
	var newValue = true; // Default
	if ((value == true) || (value == false))
		newValue = value

	this.useEmbeddedLyrics = newValue;
	document.getElementById("useEmbeddedLyricsCheck").checked = newValue;
	widgetPref.setValue(newValue, "useEmbeddedLyrics");
}

CurrentTrack.prototype.useEmbeddedLyricsCheckOnChange = function() {
	var flag = document.getElementById("useEmbeddedLyricsCheck").checked;
	this.setUseEmbeddedLyrics(flag);
}

// ================================================
// LyricsEditor
// ================================================
function LyricsEditor() {
	LOG__("Object: LyricsEditor");
	this.searchResult = new Array;
}

LyricsEditor.prototype.loadSearchResult = function(searchResult) {
	LOG__("LyricsEditor.loadSearchResult");
	var obj;
	var i;

	if (searchResult && (0 < searchResult.length)) {
		for (i=0; i < searchResult.length; i+=6) {
			obj = new Object();
			obj.title         = searchResult[i];
			obj.artist        = searchResult[i+1];
			obj.lyrics        = searchResult[i+2];
			obj.uri           = searchResult[i+3];
			obj.uriForBrowser = searchResult[i+4];
			obj.score         = searchResult[i+5];
			this.searchResult.push(obj);
		}
	}
	for (i = 0; i < this.searchResult.length; i++)
		LOG__(i + ": " + this.searchResult[i].title);
	
	this.displaySearchResult();
	currentTrack.evaluateSearchResult();
}

LyricsEditor.prototype.displaySearchResult = function() {
	LOG__("LyricsEditor.displaySearchResult");
	var list = document.getElementById("searchresult");
	var li;
	var buffer;
	var spanTitle, spanArtist, spanWebsite;
	var webSite, uri;
	var i;

	if (0 < this.searchResult.length) {
		for (i = 0; i < this.searchResult.length; i++) {
			li = document.createElement("li");

            // Song artist
			spanArtist = document.createElement("span");
			spanArtist.setAttribute("class", "sArtist");
			spanArtist.innerText = this.searchResult[i].artist
			li.appendChild(spanArtist);

            // Song title
			spanTitle = document.createElement("span");
			spanTitle.setAttribute("class", "sTitle");
			spanTitle.innerText = this.searchResult[i].title;
			li.appendChild(spanTitle);
            
            // URI
			uri = this.searchResult[i].uriForBrowser;
            if (uri.match(/^http:\/\/www\.utamap\.com\//)) webSite = "Utamap";
            if (uri.match(/^http:\/\/www\.sing365\.com\//)) webSite = "SING365.COM";
            if (uri.match(/^http:\/\/www\.metrolyrics\.com\//)) webSite = "MetroLyrics";
            
            // title attribute
            buffer =  (this.searchResult[i].artist + " - ");
            buffer += (this.searchResult[i].title + "\n");
            buffer += (getLocalizedString("Database") + " : ");
            buffer += (getLocalizedString(webSite) + "\n");
            buffer += (this.searchResult[i].lyrics);
			li.setAttribute("title", buffer);
            
            // OnClick event
			li.setAttribute("onclick", "getLyrics(event, " + i + ", \'" + uri + "\')");

			list.appendChild(li);
		}
		frontDisplay.setStatus("Lyrics found.", this.searchResult.length, false)
	} else {
		frontDisplay.setStatus("Lyrics not found.", "", false)
	}
}

LyricsEditor.prototype.clearSearchResult = function() {
	var list = document.getElementById("searchresult");

	this.searchResult.length = 0;
	while (list.hasChildNodes())
		list.removeChild(list.lastChild);
}

LyricsEditor.prototype.loadBuffer = function() {
    var tmpString;
    tmpString = document.getElementById("textView").value;
    document.getElementById("textView").value = document.getElementById("buffer").value;
    document.getElementById("buffer").value = tmpString;
}

// ================================================
// FrontDisplay
// ================================================
function FrontDisplay(track) {
	LOG__("Object: FrontDisplay");
	var divs, node, option;
	var i, len;

	this.track = track;
	this.lyrics = document.getElementById("lyrics");
	this.displayArtwork = false;
	this.setDisplay("artist");

	// Prepare style selector
	for(i = 0, len = styleCollection.items.length; i < len; i++) {
		option = document.createElement("option");
		option.innerText = styleCollection.items[i].title;
		option.value = i;
		document.getElementById("stylenum").appendChild(option);
	}

	// Status line
	this.frontStatusLine = new Fader("status", false);
	this.frontStatusLine.delay = 2000;
	this.frontStatusLine.outDuration = 6000;

	// Set properties
	this.setStyleNum(widgetPref.value("styleNum"));
	this.setFontSize(widgetPref.value("fontSize"));
	this.setDisplayArtwork(widgetPref.value("displayArtwork"), true);

    // Animator
    var localThis = this;
    this.contentPositionAnimator = null;
	this.contentPositionAnimationHandler = function(currentAnimator, current, start, finish) {
        localThis.setContentPosition(current);
    };
}

FrontDisplay.prototype.setDisplay = function(value) {
	var artistStyle = document.getElementById("artist").style;
	var albumStyle = document.getElementById("album").style;

	LOG__("FrontDisplay.setDisplay(" + value + ")");
	var newValue = value;

	if ((value == null)
			|| !((value == "artist") || (value == "album") || (value == "++")))
		newValue = "artist";

	if (value == "++") {
		if (this.display != null)
			newValue = (this.display == "artist")? "album" : "artist";
		else
			newValue = "artist";
	}

	switch(newValue) {
		case "artist":
			artistStyle.display = "block";
			albumStyle.display = "none";
			break;
		case "album":
			artistStyle.display = "none";
			albumStyle.display = "block";
			break;
	}

	this.display = newValue;
}

FrontDisplay.prototype.setTrack = function() {
	var htmlLyrics = "";
	var i;
	var table, tr, td;
	
	if (this.track.title && this.track.title != "") {
		LOG__("frontDisplay.setTrack:" + this.track.title);
	} else {
		LOG__("frontDisplay.setTrack: no track");
		return;
	}

	// Display Title and artist
	document.getElementById("title").innerText = this.track.title;
	document.getElementById("artist").innerText = this.track.artist;
	document.getElementById("album").innerText = this.track.album;
	document.getElementById("title2").innerText = this.track.title;
	document.getElementById("artist2").innerText = this.track.artist

	// Set default strings for lyrice search
	document.getElementById("searchTitle").value = this.track.title;
	document.getElementById("searchArtist").value = this.track.artist
	document.getElementById("searchTitle").setAttribute("placeholder",this.track.title);
	document.getElementById("searchArtist").setAttribute("placeholder",this.track.artist);

	// Display artwork
	this.loadArtwork()

	// Display lyrics
	if (this.track.lyrics)
		htmlLyrics = this.track.lyrics.plainToHTML();

	// Make lyrics container
	table = document.createElement("table");
	tr = document.createElement("tr");
	td = document.createElement("td");
	table.setAttribute("id", "lyricsInner");
	td.innerHTML = htmlLyrics;
	tr.appendChild(td);
	table.appendChild(tr);

	while (this.lyrics.hasChildNodes())
		this.lyrics.removeChild(this.lyrics.lastChild);
	this.lyrics.appendChild(table);

	// Resize scroller and window
	if (display == "regular") {
		if (widgetWindow.autoResize == true)
			widgetWindow.resizeToFit();
		scrollArea.verticalScrollTo(0);
		scrollArea.refresh();
	}
}

FrontDisplay.prototype.loadArtwork = function() {
	var artworkImg = document.getElementById("artworkImg");

	LOG__("FrontDisplay.loadArtwork");

	if ((this.track.artwork != null) && (this.track.artwork != ""))
        artworkImg.setAttribute("src", "file://" + this.track.artwork + "?" + new Date().getTime());
	else
        artworkImg.setAttribute("src", "Images/blankArtwork.png");
}

FrontDisplay.prototype.setDisplayArtwork = function(value, immediately) {
	LOG__("FrontDisplay.setDisplayArtwork(" + value + ")");
	var newValue = true; // Default

	if ((value == true) || (value == false))
		newValue = value;

	if (value == "++") {
		if (this.displayArtwork != null)
			newValue = (this.displayArtwork == true)? false : true;
	}

	if (newValue) {
        if (immediately) {
            this.setContentPosition(1);
        } else {
            if (this.contentPositionAnimator) this.contentPositionAnimator.stop();
            this.contentPositionAnimator = new AppleAnimator(250, 13, 0, 1, this.contentPositionAnimationHandler);
            this.contentPositionAnimator.start();
        }
	} else {
        if (immediately) {
            this.setContentPosition(0);
        } else {
            if (this.contentPositionAnimator) this.contentPositionAnimator.stop();
            this.contentPositionAnimator = new AppleAnimator(250, 13, 1, 0, this.contentPositionAnimationHandler);
            this.contentPositionAnimator.start();
        }
    }

	this.displayArtwork = newValue;
	widgetPref.setValue(newValue, "displayArtwork");
}

FrontDisplay.prototype.setContentPosition = function(value) {
	LOG__("FrontDisplay.setContentPosition(" + value + ")");
	var artwork     = document.getElementById("artwork");
	var artworkImg  = document.getElementById("artworkImg");
	var contentBox  = document.getElementById("contentBox");
	var myScrollBar = document.getElementById("myScrollBar");
	var newValue = 0; // Default
    
    newValue = ((0 <= value) && (value <= 1))? value : 0;

    artwork.style.opacity = Math.sqrt(newValue);
    artworkImg.style.marginTop = 6 - 150 * (1 - newValue) + "px";
    contentBox.style.top = 42 + 150 * newValue + "px";
    myScrollBar.style.top = 50 + 150 * newValue + "px";

	if (widgetWindow && ((newValue = 0) || (newValue = 1))) {
		widgetWindow.setMinHeight(130 + 150 * newValue);
		if (widgetWindow.autoResize)
			widgetWindow.resizeToFit();
		scrollArea.verticalScrollTo(0);
		scrollArea.refresh();
	}
}

FrontDisplay.prototype.setFontSize = function(value) {
	LOG__("FrontDisplay.setFontSize(" + value + ")");
	var newValue = 0; // Default

	if (value == (Math.floor(value)) && (-2 <= value) && (value <= 3))
		newValue = value;

	if (value == "++") {
		if (this.fontSize != null)
			newValue = (this.fontSize < 3)? this.fontSize + 1 : 3;
	}

	if (value == "--") {
		if (this.fontSize != null)
			newValue = (-2 < this.fontSize)? this.fontSize - 1 : -2;
	}

	switch (newValue) {
		case -2:
//			document.getElementById("lyrics").style.fontSize = "7pt";
			document.getElementById("lyrics").style.fontSize = "8pt";
			break;
		case -1:
//			document.getElementById("lyrics").style.fontSize = "8pt";
			document.getElementById("lyrics").style.fontSize = "9pt";
			break;
		case 1:
//			document.getElementById("lyrics").style.fontSize = "11pt";
			document.getElementById("lyrics").style.fontSize = "12pt";
			break;
		case 2:
			document.getElementById("lyrics").style.fontSize = "14pt";
			break;
		case 3:
			document.getElementById("lyrics").style.fontSize = "18pt";
			break;
		default:
//			document.getElementById("lyrics").style.fontSize = "9pt";
			document.getElementById("lyrics").style.fontSize = "11pt";
			break;
	}

	scrollArea.refresh();
	this.fontSize = newValue;
	document.getElementById("fontsize").selectedIndex = newValue + 2;
	widgetPref.setValue(newValue, "fontSize");
}

FrontDisplay.prototype.setStyleNum = function(value) {
	LOG__("FrontDisplay.setStyleNum(" + value + ")");
	var i;
	var len = styleCollection.items.length;
	var linkCSS = document.getElementById("linkCSS");
	var infoButton = document.getElementById("infobutton");
	var newValue = 0; // Default

	if ((value == Math.floor(value)) && (0 <= value) && (value < len))
		newValue = value;

	if (value == "++") {
		if (this.styleNum != null)
			newValue = (this.styleNum < len - 1)? this.styleNum + 1 : 0;
	}

	if (value == "--") {
		if (this.styleNum != null)
			newValue = (0 < this.styleNum)? this.styleNum - 1 : len - 1;
	}

	if (this.styleNum == newValue) return;

	linkCSS.setAttribute("href", styleCollection.items[newValue].href);

	if (display == "rgular")
		scrollArea.refresh();
	this.styleNum = newValue;
	this.setStatus("Style", styleCollection.items[newValue].title, false);

	// change info button color
	switch (styleCollection.items[newValue].buttonColor) {
		case "black":
			infoButton.object.setStyle("black", "black");
			scrollBar.setTrackStart(
				"AppleClasses/Images/scroll_track_vtop.png", scrollBar.trackStartLength);
			scrollBar.setTrackMiddle(
				"AppleClasses/Images/scroll_track_vmid.png");
			scrollBar.setTrackEnd(
				"AppleClasses/Images/scroll_track_vbottom.png", scrollBar.trackEndLength);
			scrollBar.setThumbStart(
				"AppleClasses/Images/scroll_thumb_vtop.png", scrollBar.thumbStartLength);
			scrollBar.setThumbMiddle(
				"AppleClasses/Images/scroll_thumb_vmid.png");
			scrollBar.setThumbEnd(
				"AppleClasses/Images/scroll_thumb_vbottom.png", scrollBar.thumbEndLength);
			break;
		case "white":
		default:
			infoButton.object.setStyle("white", "white");
			scrollBar.setTrackStart(
				"Images/scroll_track_vtop.png", scrollBar.trackStartLength);
			scrollBar.setTrackMiddle(
				"Images/scroll_track_vmid.png");
			scrollBar.setTrackEnd(
				"Images/scroll_track_vbottom.png", scrollBar.trackEndLength);
			scrollBar.setThumbStart(
				"Images/scroll_thumb_vtop.png", scrollBar.thumbStartLength);
			scrollBar.setThumbMiddle(
				"Images/scroll_thumb_vmid.png");
			scrollBar.setThumbEnd(
				"Images/scroll_thumb_vbottom.png", scrollBar.thumbEndLength);
			break;
	}

	document.getElementById("stylenum").selectedIndex = newValue;
	widgetPref.setValue(newValue, "styleNum");
}

FrontDisplay.prototype.setStatus = function(str, prm, important) {
	var i;
	var message;
	LOG__("setStatus: " + str);

	message = getLocalizedString(str);
	if (prm != "")
		message = message + " : " + prm;

	// Make a hyperlink
	message = message.replace("TunesTEXT", "<span onclick='openWebsite(websiteURI);' style='text-decoration: underline; cursor: pointer;'>TunesTEXT</span>");

	document.getElementById("status").innerHTML = message;
	document.getElementById("status2").innerHTML = message;

	if (message != "") {
		this.frontStatusLine.setOpacity(0.75);
		if (!important)
			this.frontStatusLine.fadeOut();
	} else {
		this.frontStatusLine.setOpacity(0);
	}
}

// Open website
function openWebsite(uri) {
	if (window.widget)
		widget.openURL(uri);
	else
		window.location.href = uri;
}

function setDisplay(value) {
	LOG__("setDisplay(" + value + ")");
	var newValue = "regular";
	var contentsRegular = document.getElementById("contentsRegular");
	var contentsCompact = document.getElementById("contentsCompact");

	if ((value == "regular") || (value == "compact"))
		newValue = value;

	if (value == "++") {
		if (display != null)
			newValue = (display == "regular")? "compact" : "regular";
	}

	switch(newValue) {
		case "regular":
			contentsRegular.style.visibility = "visible";
			contentsCompact.style.visibility = "hidden";
			if (window.widget)
				window.resizeTo(widgetWindow.width, widgetWindow.height);
			if (widgetWindow.autoResize == true)
				widgetWindow.resizeToFit();
			break;
		case "compact":
			contentsRegular.style.visibility = "hidden";
			contentsCompact.style.visibility = "visible";
			if (window.widget)
				window.resizeTo(widgetWindow.width, 55);
			break;
	}

	display = newValue;
	widgetPref.setValue(newValue, "display");
}

function setBacksidePane(value) {
	LOG__("setBacksidePane(" + value + ")");
	var newValue = "pref";
	var prefPane   = document.getElementById("prefPane");
	var editorPane = document.getElementById("editorPane");
	var showPrefPane   = document.getElementById("showprefpane");
	var showEditorPane = document.getElementById("showeditorpane");

	if ((value == "pref") || (value == "editor"))
		newValue = value;

	switch(newValue) {
		case "pref":
			prefPane.style.display   = "block";
            editorPane.style.display = "none";
			showPrefPane.className   = "prefpanetab active";
            showEditorPane.className = "prefpanetab inactive";
			break;
		case "editor":
            prefPane.style.display   = "none";
			editorPane.style.display = "block";
            showPrefPane.className   = "prefpanetab inactive";
			showEditorPane.className = "prefpanetab active";
			break;
	}

	backsidePane = newValue;
	widgetPref.setValue(newValue, "backsidePane");
}

// Dispatch the mouseout event (hide buttons when widget is flipped)
function simulateMouseOut() {
	var front = document.getElementById("front");
	var infoButton = document.getElementById("infobutton");
	var event = document.createEvent("MouseEvents");
	event.initEvent("mouseout", true, true);

	infoButton.dispatchEvent(event);
	front.dispatchEvent(event);
}

// Reverse sides and preferences
function showBack(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.outerWidth < widgetWindow.widthRev)
		window.resizeBy(widgetWindow.widthRev - window.outerWidth, 0);

	if (window.outerHeight < widgetWindow.heightRev)
		window.resizeBy(0, widgetWindow.heightRev - window.outerHeight);

	if (window.widget)
		widget.prepareForTransition("ToBack");

	setTimeout ("simulateMouseOut();", 0);
	front.style.display = "none";
	back.style.display = "block";
	display = "backside";
	scrollArea.blur();

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
		
	// Prepare a save reminder
	setTimeout ("saveReminder.reset();", 0);
}

function showFront(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	// Save reminder
	if (saveReminder.check())
		return;

	if (window.outerWidth < widgetWindow.width)
		window.resizeBy(widgetWindow.width - window.outerWidth, 0);
	if (window.outerHeight < widgetWindow.height)
		window.resizeBy(0, widgetWindow.height - window.outerHeight);

	if (window.widget)
		widget.prepareForTransition("ToFront");

	TunesTEXTPlugin.cancel();

	// Save preferences
	frontDisplay.setStyleNum(document.getElementById("stylenum").selectedIndex);
	frontDisplay.setFontSize(document.getElementById("fontsize").value - 0);
	widgetWindow.setAutoResize(document.getElementById("autoResizeCheck").checked);

	back.style.display = "none";
	front.style.display = "block";
	display = "regular";
	scrollArea.focus();
	scrollArea.refresh();

	currentTrack.update(true, null, null);

	if (window.widget)
		setTimeout ("widget.performTransition();", 0);
}

// Simple Base64 Decoding
function base64Decode(b64string) {
	var b64Alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
	var inBytes = new Array(4);
	var outBytes = new Array(3);
	var result = new Array();
	var i;

	for (i = 0; i < b64string.length; i += 4) {
		inBytes[0] = b64Alphabet.indexOf(b64string.charAt(i));
		inBytes[1] = b64Alphabet.indexOf(b64string.charAt(i + 1));
		inBytes[2] = b64Alphabet.indexOf(b64string.charAt(i + 2));
		inBytes[3] = b64Alphabet.indexOf(b64string.charAt(i + 3));

		// Convert 4 concatenated 6-bit groups to 3 8-bit groups
		//     IN  : 00ABCDEF 00GHIJKL 00MNOPQR 00STUVWX
		//     OUT : ABCDEFGH IJKLMNOP QRSTUVWX
		outBytes[0] = (inBytes[0] << 2) | (inBytes[1] >> 4);
		outBytes[1] = ((inBytes[1] & 0x0f) << 4) | ((inBytes[2] & 0x3c) >> 2);
		outBytes[2] = ((inBytes[2] & 0x03) << 6) | (inBytes[3] & 0x3f);
		result.push(outBytes[0], outBytes[1], outBytes[2]);
	}
	return result;
}

String.prototype.plainToHTML = function() {
	var str = new String(this);
	str = str.replace(/</g, "&lt;");
	str = str.replace(/>/g, "&gt;");
	str = str.replace(/\r\n/g, "<br/>");    // CR + LF
	str = str.replace(/\r/g, "<br/>");      // CR
	str = str.replace(/\n/g, "<br/>");      // LF
	str = str.replace(/\u2028/g, "<br/>");  // Unicode Line Separator (U+2028)
	str = str.replace(/\u2029/g, "<br/>");  // Unicode Paragraph Separator (U+2029)
//	str = str.replace(/\u0020/g, "___NBSP___"); // Space to No-break Space (U+00A0)
//	str = str.replace(/\t/g, "\u00a0\u00a0\u00a0\u00a0\u00a0\u00a0\u00a0\u00a0");  // Tab to Spaces
	str = str.replace(/^(<br\/>){1,5}/, "");
	str = str.replace(/(<br\/>){1,5}$/, "");
	return str;
}

String.prototype.plainToForm = function() {
	var str = new String(this);
	str = str.replace(/\u2028/g, "\r");  // Unicode Line Separator (U+2028)
	str = str.replace(/\u2029/g, "\r");  // Unicode Paragraph Separator (U+2029)
	return str;
}

// Localizing
function localizeInnerText() {
    var localElms = ["div", "span", "option"];
    var localText;
    var key;
    var i, j;

    for (i = 0; i < localElms.length; i++) {
        localText = document.getElementsByTagName(localElms[i]);
        for (j = 0; j < localText.length; j++) {
            if (localText[j].className.match("localText")) {
                key = localText[j].innerHTML;
                localText[j].innerHTML = getLocalizedString(key);
            }
        }
    }

    /*
    localText = document.getElementsByClassName("localText");
    for (i = 0; i < localText.length; i++) {
        key = localText[i].innerText;
        localText[i].innerText = getLocalizedString(key);
    }
    */
}

// ================================================
// Debugging
// ================================================
function LOG__(arg) {
//	alert(arg);
}

function LOG2__(arg) {
//	alert(arg);
}

function LOG3__(arg) {
//	alert(arg);
}
