// ================================================
// UpdateChecker
// ================================================

function UpdateChecker() {
	LOG__("Object: UpdateChecker");

	this.myInfoURI = "./info.plist";
	this.manifestURI = "http://www.cyanworks.net/updateManifest.plist";
	this.myBundleID = "net.cyanworks.widget.widgetName";
	this.myVersion = null;
	this.latestVersion = null;
	this.checkFinished = function() {};
	this.timer = null;
	this.timerInterval =    15*60*1000;  // 15 minutes
	this.checkInterval = 24*60*60*1000;  // 1 day
	this.lastCheckedTime = 0;
	this.disableMe = false;
	
	// TEST
	/*
	this.manifestURI = "http://127.0.0.1/~cyan/updateManifest.plist";
	this.timerInterval = 5*1000;
	this.checkInterval = 60*1000;
	*/

	// OS X 10.4 seems does not support XPath
	if (typeof XPathResult == "undefined")
		this.disableMe = true

	// Run the timer
	this.start = function() {
		LOG2__("UpdateChecker.start");
		
		var localThis = this;

		if (this.disableMe)
			return;

		if (this.timer != null)
			clearInterval(this.timer);

		// Initial event
		setTimeout(function() { localThis.timerEvent(); }, 3*1000);

		// Interval event
		this.timer = setInterval(function() { localThis.timerEvent(); }, this.timerInterval);
	}

	this.stop = function() {
		LOG2__("UpdateChecker.stop");
		
		if (this.timer != null) {
			clearInterval(this.timer);
			this.timer = null;
		}
	}

	// Interval event
	this.timerEvent = function() {
		LOG2__("UpdateChecker.timerEvent");
		
		var now = (new Date).getTime();
		var result = this.judge();
		
		// If we alreday know that the update is available, that's enough.
		if (result == -1) {
			this.checkFinished();
			return;
		}

		if (this.checkInterval <= (now - this.lastCheckedTime))
			this.check();
	}

	// Check for updates
	this.check = function() {
		LOG2__("UpdateChecker.check");
		
		var now = (new Date).getTime();

		this.lastCheckedTime = now;
		this.loadManifest();
	}

	this.judge = function() {
		LOG2__("UpdateChecker.judge");

		// -1: The update is available
		//  0: There are no Updates
		//  1: Illegal version number

		if ((this.myVersion != null) && (this.latestVersion != null))
			return compareVersionString(this.myVersion, this.latestVersion)
		else
			return 1;
	}
	
	// Load info.plist of this widget
	this.loadMyInfo = function() {
		LOG2__("UpdateChecker.loadMyInfo");

		var httpReq;
		var xml;
		var xPath;
		var bundleID;
		var version;
	
		if (this.disableMe)
			return;
		
		// XMLHttpRequest (Synchronous)
		httpReq = new XMLHttpRequest()
		httpReq.open("GET", this.myInfoURI, false);
		httpReq.send(null);

		// Read XML
		if (httpReq.status == 0) {
			xml = httpReq.responseXML;

			// Get CFBundleIdentifier
			xPath = "/plist/dict/string[preceding-sibling::key[1] = 'CFBundleIdentifier']";
			bundleID = xml.evaluate(xPath, xml, null, XPathResult.STRING_TYPE, null);
			this.myBundleID = bundleID.stringValue;

			// Get CFBundleVersion
			xPath = "/plist/dict/string[preceding-sibling::key[1] = 'CFBundleVersion']";
			version = xml.evaluate(xPath, xml, null, XPathResult.STRING_TYPE, null);
			this.myVersion = version.stringValue;
		}
	}
	

	// Download the update manifest file
	this.loadManifest = function() {
		LOG2__("UpdateChecker.loadManifest");
	
		var localThis = this;

		if (this.disableMe)
			return;

		// XMLHttpRequest
		this.httpReq = new XMLHttpRequest()
		this.httpReq.onload = function() { localThis.onLoadManifest(); };
		this.httpReq.open("GET", this.manifestURI +  "?" + new Date().getTime(), true);
		this.httpReq.send(null);
	}
	

	// On load the update manifest file
	this.onLoadManifest = function() {
		LOG2__("UpdateChecker.onLoadManifest");
		var xml;
		var nodes;
		var xPath;
		var dict;
		var version;
	
		xPath = "/plist/dict/array/dict[./string[preceding-sibling::key[1] = 'CFBundleIdentifier']='" + this.myBundleID +"']";
	
		if (this.httpReq.status == 200) {
			xml = this.httpReq.responseXML;

			if (xml != null) {
				// Find dict element
				xPath = "/plist/dict/array/dict[./string[preceding-sibling::key[1] = 'CFBundleIdentifier']='" + this.myBundleID +"']";
				nodes = xml.evaluate(xPath, xml, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			}
			
			if ((nodes != null) && (nodes.snapshotLength == 1)) {
				// Get CFBundleVersion
				dict = nodes.snapshotItem(0);
				xPath = "./string[preceding-sibling::key[1] = 'CFBundleVersion']";
				version = xml.evaluate(xPath, dict, null, XPathResult.STRING_TYPE, null);
				if (version != null)
					this.latestVersion = version.stringValue;
			}
		}
		
		LOG__("my version : " + this.myVersion + "  " + "latest version : " + this.latestVersion);
		this.checkFinished();
	}

	// Load info.plist of this widget
	this.loadMyInfo();
}


// Compare two version numbers
function compareVersionString(verA, verB) {
	var numsA;
	var numsB;
	var idx;
	var idxMax;

	if ((typeof(verA) != 'string') || (typeof(verB) != 'string')) 
		return null;

	numsA = verA.split(".");
	numsB = verB.split(".");

	for (idx = 0; idx < numsA.length; idx++) {
		if ((numsA[idx] - 0) < (numsB[idx] - 0))
			return -1;
		if ((numsA[idx] - 0) > (numsB[idx] - 0))
			return 1;
	}

	if (numsA.length < numsB.length)
		return -1;

	return 0;
}